@@ -73,7 +73,7 @@ static int enable_tilde_expansion;
 
 CMD_DECLARE (cmd_boolean);
 CMD_DECLARE (cmd_bytes);
-CMD_DECLARE (cmd_bytes_large);
+CMD_DECLARE (cmd_bytes_sum);
 #ifdef HAVE_SSL
 CMD_DECLARE (cmd_cert_type);
 #endif
@@ -180,7 +180,6 @@ static struct {
 #endif
   { "input",		&opt.input_filename,	cmd_file },
   { "keepsessioncookies", &opt.keep_session_cookies, cmd_boolean },
-  { "killlonger",	&opt.kill_longer,	cmd_boolean },
   { "limitrate",	&opt.limit_rate,	cmd_bytes },
   { "loadcookies",	&opt.cookies_input,	cmd_file },
   { "logfile",		&opt.lfilename,		cmd_file },
@@ -210,7 +209,7 @@ static struct {
   { "proxypassword",	&opt.proxy_passwd,	cmd_string },
   { "proxyuser",	&opt.proxy_user,	cmd_string },
   { "quiet",		&opt.quiet,		cmd_boolean },
-  { "quota",		&opt.quota,		cmd_bytes_large },
+  { "quota",		&opt.quota,		cmd_bytes_sum },
 #ifdef HAVE_SSL
   { "randomfile",	&opt.random_file,	cmd_file },
 #endif
@@ -572,7 +571,7 @@ parse_line (const char *line, char **com, char **val, int *comind)
   p = line;
 
   cmdstart = p;
-  while (p < end && (ISALPHA (*p) || *p == '_' || *p == '-'))
+  while (p < end && (ISALNUM (*p) || *p == '_' || *p == '-'))
     ++p;
   cmdend = p;
 
@@ -704,7 +703,7 @@ cmd_boolean (const char *com, const char *val, void *place)
   else
     {
       fprintf (stderr,
-	       _("%s: %s: Invalid boolean `%s', use `on' or `off'.\n"),
+	       _("%s: %s: Invalid boolean `%s'; use `on' or `off'.\n"),
 	       exec_name, com, val);
       return 0;
     }
@@ -721,7 +720,16 @@ cmd_boolean (const char *com, const char *val, void *place)
    Values: 2 - Enable a particular option for good ("always")
            1 - Enable an option ("on")
            0 - Disable an option ("off")
-          -1 - Disable an option for good ("never") */
+          -1 - Disable an option for good ("never")
+
+   #### This hack is currently only used for passive FTP because a
+   contributor had broken scripts specify --passive-ftp where he
+   didn't want it.  It should be removed because the same can now be
+   achieved by replacing the wget executable with a script containing:
+
+       exec wget "$@" --no-passive-ftp
+*/
+
 static int
 cmd_lockable_boolean (const char *com, const char *val, void *place)
 {
@@ -736,18 +744,19 @@ cmd_lockable_boolean (const char *com, const char *val, void *place)
   if (oldval == -1 || oldval == 2)
     return 1;
 
-  if (0 == strcasecmp (val, "always") || CMP1 (val, '2'))
-    lockable_boolean_value = 2;
-  else if (CMP2 (val, 'o', 'n') || CMP3 (val, 'y', 'e', 's') || CMP1 (val, '1'))
+  if (CMP2 (val, 'o', 'n') || CMP3 (val, 'y', 'e', 's') || CMP1 (val, '1'))
     lockable_boolean_value = 1;
   else if (CMP3 (val, 'o', 'f', 'f') || CMP2 (val, 'n', 'o') || CMP1 (val, '0'))
     lockable_boolean_value = 0;
-  else if (0 == strcasecmp (val, "never") || CMP2 (val, '-', '1'))
+  else if (0 == strcasecmp (val, "always"))
+    lockable_boolean_value = 2;
+  else if (0 == strcasecmp (val, "never"))
     lockable_boolean_value = -1;
   else
     {
       fprintf (stderr,
-	       _("%s: %s: Invalid boolean `%s', use always, on, off, or never.\n"),
+	       _("%s: %s: Invalid extended boolean `%s';\n\
+use one of `on', `off', `always', or `never'.\n"),
 	       exec_name, com, val);
       return 0;
     }
@@ -918,7 +927,7 @@ cmd_directory_vector (const char *com, const char *val, void *place)
   return 1;
 }
 
-/* Engine for cmd_bytes and cmd_bytes_large: converts a string such as
+/* Engine for cmd_bytes and cmd_bytes_sum: converts a string such as
    "100k" or "2.5G" to a floating point number.  */
 
 static int
@@ -1004,12 +1013,12 @@ cmd_bytes (const char *com, const char *val, void *place)
 }
 
 /* Like cmd_bytes, but PLACE is interpreted as a pointer to
-   LARGE_INT.  It works by converting the string to double, therefore
+   SIZE_SUM.  It works by converting the string to double, therefore
    working with values up to 2^53-1 without loss of precision.  This
    value (8192 TB) is large enough to serve for a while.  */
 
 static int
-cmd_bytes_large (const char *com, const char *val, void *place)
+cmd_bytes_sum (const char *com, const char *val, void *place)
 {
   double byte_value;
   if (!parse_bytes_helper (val, &byte_value))
@@ -1018,7 +1027,7 @@ cmd_bytes_large (const char *com, const char *val, void *place)
 	       exec_name, com, val);
       return 0;
     }
-  *(LARGE_INT *)place = (LARGE_INT)byte_value;
+  *(SUM_SIZE_INT *) place = (SUM_SIZE_INT) byte_value;
   return 1;
 }
 
